/*
 * BMP180.c
 *
 *  Created on: 01.09.2014
 *      Author: DL9MWE
 */

#include "main.h"

bmp180_t bmp180;	// BMP180-Datenstruktur



unsigned int I2C_read_16Bit_Register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int *error_code) {
	unsigned char data[2];
	*error_code = iI2C_read_multiple_chars(I2C_device_address, I2C_register_address, 2, data);
	return (((int)data[0]<<8) | (int)data[1]);
}

// Temperatur-Messung starten und Roh-Messwert zurckgeben
unsigned int uibmp180_readTemp(unsigned int *error_code) {
  unsigned int ut;

	// Temperaturmessung starten
	*error_code = iI2C_write_register_value(BMP180_I2CADDRESS, BMP180_CTRL_MEAS_REG, BMP180_T_MEASURE);
	_delay_ms(5);
	ut = I2C_read_16Bit_Register(BMP180_I2CADDRESS, BMP180_ADC_OUT_MSB_REG, error_code);
//	ut = 27898;		// Beispieldaten aus dem Datenblatt
	return ut;
}

// Druck-Messung starten und Roh-Messwert zurckgeben
long int libmp180Readup(unsigned int *error_code) {
  long int up = 0;
  unsigned char xlsb;
  unsigned char wait_ms;
	switch (bmp180.oversampling_setting) {
		case 0: wait_ms =  5; break;
		case 1: wait_ms =  8; break;
		case 2: wait_ms = 14; break;
		case 3: wait_ms = 26; break;
	}
	// Druckmessung starten
	*error_code = iI2C_write_register_value(BMP180_I2CADDRESS, BMP180_CTRL_MEAS_REG, BMP180_P_MEASURE + (bmp180.oversampling_setting << 6));
	_delay_ms(wait_ms);
	up = I2C_read_16Bit_Register(BMP180_I2CADDRESS, BMP180_ADC_OUT_MSB_REG, error_code);
	xlsb = ucI2C_read_8Bit_Register(BMP180_I2CADDRESS, BMP180_ADC_OUT_XLSB_REG, error_code);
	up = (up<<8 + xlsb) >> (8 - bmp180.oversampling_setting);
//	up = 23843;		// Beispieldaten aus dem Datenblatt
	return up;
}

// aus dem Rohwert mittels der Kalibrierungsdaten die Temperatur [C*10] ermitteln
int bmp180Calculate_temperature(int ut) {
  int temperature;
  long x1,x2;
	x1 = (((long) ut - (long) bmp180.cal_param.ac6) * (long) bmp180.cal_param.ac5) >> 15;
	x2 = ((long) bmp180.cal_param.mc << 11) / (x1 + bmp180.cal_param.md);
	bmp180.param_b5 = x1 + x2;
	temperature = ((bmp180.param_b5 + 8) >> 4);  // temperature in 0.1C
	return (temperature);
}

// aus dem Rohwert mittels der Kalibrierungsdaten den Luftdruck [Pa] ermitteln
// siehe BMP180 Datenblatt
long bmp180Calculate_pressure(long up) {
  long pressure,x1,x2,x3,b3,b6;
  unsigned long b4, b7;

	   b6 = bmp180.param_b5 - 4000;
	   //*****calculate B3************
	   x1 = (b6*b6) >> 12;
	   x1 *= bmp180.cal_param.b2;
	   x1 >>=11;

	   x2 = (bmp180.cal_param.ac2*b6);
	   x2 >>=11;

	   x3 = x1 +x2;

	   b3 = (((((long)bmp180.cal_param.ac1 )*4 + x3) << bmp180.oversampling_setting) + 2) >> 2;

	   //*****calculate B4************
	   x1 = (bmp180.cal_param.ac3* b6) >> 13;
	   x2 = (bmp180.cal_param.b1 * ((b6*b6) >> 12) ) >> 16;
	   x3 = ((x1 + x2) + 2) >> 2;
	   b4 = (bmp180.cal_param.ac4 * (unsigned long) (x3 + 32768)) >> 15;

	   b7 = ((unsigned long)(up - b3) * (50000>>bmp180.oversampling_setting));
	   if (b7 < 0x80000000)
	   {
	     pressure = (b7 << 1) / b4;
	   }
	   else
	   {
	     pressure = (b7 / b4) << 1;
	   }

	   x1 = pressure >> 8;
	   x1 *= x1;
	   x1 = (x1 * /*SMD500_PARAM_MG*/ 3038) >> 16;
	   x2 = (pressure * /*SMD500_PARAM_MH*/ -7357) >> 16;
	   pressure += (x1 + x2 + /*SMD500_PARAM_MI*/ 3791) >> 4;	// pressure in Pa

	   return (pressure);
}



// ------------------------------ BMP180 API ------------------------------

// BMP180 initialisieren: Kalibrierungsdaten lesen, Voreinstellungen setzen
int iBMP180_init(void) {
	int temp = 0;
	unsigned int error_code;
// eine for-Schleife knnte hier ein paar Bytes sparen...
	bmp180.cal_param.ac1 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xAA, &error_code);	//ac1
	bmp180.cal_param.ac2 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xAC, &error_code);	//ac2
	bmp180.cal_param.ac3 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xAE, &error_code);	//ac3
	bmp180.cal_param.ac4 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xB0, &error_code);	//ac4
	bmp180.cal_param.ac5 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xB2, &error_code);	//ac5
	bmp180.cal_param.ac6 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xB4, &error_code);	//ac6
	bmp180.cal_param.b1 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xB6, &error_code);	//b1
	bmp180.cal_param.b2 = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xB8, &error_code);	//b2
	bmp180.cal_param.mb = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xBA, &error_code);	//mb
	bmp180.cal_param.mc = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xBC, &error_code);	//mc
	bmp180.cal_param.md = I2C_read_16Bit_Register(BMP180_I2CADDRESS, 0xBE, &error_code);	//md
// ...

	bmp180.chip_id = ucI2C_read_8Bit_Register(BMP180_I2CADDRESS, BMP180_CHIP_ID_REG, &error_code);
	bmp180.number_of_samples = 1;
	bmp180.oversampling_setting=0;
	temp = I2C_read_16Bit_Register(BMP180_I2CADDRESS, BMP180_VERSION_REG, &error_code);
	bmp180.al_version = temp & 0xFF;
	bmp180.ml_version = (temp>>8) & 0xFF;
	return error_code;
}

// Temperatur-Messwert zurckgeben
int iBMP180_get_temperature(unsigned int* error_code) {
  int ut;
  ut = uibmp180_readTemp(error_code);
//	ut = bmp180Readut(error_code);
	return bmp180Calculate_temperature(ut);
}

// Druck-Messwert zurckgeben
long int liBMP180_get_pressure(unsigned int* error_code) {
  long int up;
	up = libmp180Readup(error_code);
	return bmp180Calculate_pressure(up);
}

// Ende BMP180.c



