/*
 * BMP180.h
 *
 *  Created on: 01.09.2014
 *      Author: M
 */

#ifndef BMP180_H_
#define BMP180_H_


// Module Address
#define BMP180_I2CADDRESS 0xEE	// I2C-Adresse des BMP180 (fr Master-Write; 0xEF ^= Read)

// BMP180 Register Adressen und Control Register Values
#define BMP180_CHIP_ID_REG			0xD0
#define BMP180_VERSION_REG			0xD1
#define BMP180_CTRL_MEAS_REG		0xF4  // measurement Register Address
#define BMP180_T_MEASURE        	0x2E	// Control Register Value, temperature measurent
#define BMP180_P_MEASURE        	0x34	// Control Register Value, pressure measurement
#define BMP180_ADC_OUT_MSB_REG		0xF6
#define BMP180_ADC_OUT_LSB_REG		0xF7
#define BMP180_ADC_OUT_XLSB_REG		0xF8


// API
int iBMP180_init(void);						// Kalibrierungsdaten etc. lesen
int iBMP180_get_temperature(unsigned int *error_code);	// Temperaturwert ermitteln
long int liBMP180_get_pressure(unsigned int *error_code);		// Druckwert ermitteln


/*
 * this structure holds all device specific calibration parameters
*/
typedef struct {
   int ac1;
   int ac2;
   int ac3;
   unsigned int ac4;
   unsigned int ac5;
   unsigned int ac6;
   int b1;
   int b2;
   int mb;
   int mc;
   int md;
} bmp180_calibration_param_t;


/*
 * BMP180 image registers data structure
*/
typedef struct  {
	bmp180_calibration_param_t cal_param;
	unsigned char mode;
	unsigned char chip_id,	ml_version,	al_version;
//	unsigned char sensortype;

	long param_b5;
	int number_of_samples;
	int oversampling_setting;
} bmp180_t;


extern bmp180_t bmp180;	// BMP180-Datenstruktur


#endif /* BMP180_H_ */
