/*
 * MSP430_SWI2C_Master.h
 *
 */

#ifndef I2C_H_
#define I2C_H_


//******************************************************************************
//  MSP430F20xx - I2C Master Transmitter and Receiver via CPU Software (GPIO)
//
//  Description: This code library configures the MSP430 as an I2C master device
//  capable of transmitting and receiving bytes using GPIO pins.  Specific I/O
//  pins can be selected in the corresponding header file.  By default, the same
//  pins that are used by the USI module are selected.
//
//                  Master
//                  MSP430
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |                 |
//            |                 |
//            |         SDA/P2.7|-------> [I2C SLAVE SDA]
//            |         SCL/P2.6|-------> [I2C SLAVE SCL]
//
//  Note: Internal pull-ups are NOT used for SDA & SCL [DISABLED]
//
//  R. Wu
//  Texas Instruments Inc.
//  March 2008
//
// gendert fr G2553 (Port 1)
//******************************************************************************

#include <msp430G2553.h>

#define PxSEL P1SEL // Port selection
#define PxDIR P1DIR // Port direction
#define PxOUT P1OUT // Port output
#define PxIN  P1IN  // Port input

#define SDA   BIT7 // Controls SDA line (pull-up used for logic 1)
#define SCL   BIT6 // Controls SCL line (pull-up used for logic 1)


// Prototypes
void vI2C_init(void);
int iI2C_write_register_value(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned char register_value);
int iI2C_read_multiple_chars(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int numBytes, unsigned char* rxData);
unsigned char ucI2C_read_8Bit_Register(unsigned char I2C_device_address, unsigned char I2C_register_address, unsigned int *error_code);

//#define	I2C_LSBFIRST	0
//#define	I2C_MSBFIRST	1

#endif /* I2C_H_ */
