/*
 * LCD.c
 *
 *  Created on: 25.07.2012
 *      Author: Gerrit
 *  abgespeckte und fr 1MHz-DCO-Takt optimierte Version, 31.08.2014, DL9MWE
 */

#include "main.h"

//unsigned char ucLCD_x, ucLCD_y, ucLCD_maxx;
unsigned int const uiBase_y[4] = { 0x80, 0xc0, 0x94, 0xd4 }; // example for 4x20 display
// 1. line 0x00..0x13 + Bit7==0x80
// 2. line 0x40..0x53 + Bit7==0xc0
// 3. line 0x14..0x27 + Bit7==0x94
// 4. line 0x54..0x67 + Bit7==0xd4
// BIT7 nessesary for command "Set DD RAM address"

//********************************************************************************
// Description  : delay for >=50us
// Input        : none
// Output       : none
//********************************************************************************
void vLCD_delay(unsigned int delay_50) {
	volatile unsigned int i;
	do  {
		for (i = 5; i > 0; i--)	;	// passt ganz gut bei 1MHz clock
	} while(--delay_50);

}

//********************************************************************************
//Description  : Sends byte to lcd in 4bit-mode
//Input        : data - data to send
//             : reg - specifies the lcd-register
//Output       : none
//********************************************************************************
void vSendByteToLCD_4BIT(unsigned char ucData, unsigned char ucReg) {
// Data valid at HIGH-LOW transition of enable signals !!
// High nibble (BIT7-BIT4) are transfered first
   if (ucReg == COMMAND)	SELECT_COMMAND_REG;
   else						SELECT_DATA_REG;

   //send high nibble of data to LCD
   DISPLAY_ENABLE_HIGH;
   LCD_DATA_OUT &= ~DATA_LINE_BITMASK;	//clear only nibble which is connected to LCD, remain other nibble untouched!
   if (DATA_LINE_BITMASK == 0x0f)		//is low nibble of port used?
      LCD_DATA_OUT |= ucData>>4;		//high nibble remains untouched!
   else
      LCD_DATA_OUT |= ucData & 0xf0;	//lower nibble remains untouched!
   vLCD_delay(1);
   DISPLAY_ENABLE_LOW;
   vLCD_delay(1);

   //send low nibble of data to LCD
   DISPLAY_ENABLE_HIGH;
   LCD_DATA_OUT &= ~DATA_LINE_BITMASK;	//clear only nibble which is connected to LCD, remain other nibble untouched!
   if (DATA_LINE_BITMASK == 0x0f)
      LCD_DATA_OUT |= ucData & 0x0f; 	//high nibble remains untouched!
   else
      LCD_DATA_OUT |= ucData<<4 ; 		//lower nibble remains untouched!
   vLCD_delay(1);
   DISPLAY_ENABLE_LOW;
   vLCD_delay(1);
}

// ********************************************************************************
// Description  : Sends byte to the LCD's instruction-register in 8bit-mode
// Input        : data - data to send
// Output       : none
// ********************************************************************************
void vSendByteToLCD_8BIT(unsigned char ucData) {
// Data valid at HIGH-LOW transition of enable signals !!
// for this function only control-registers are written
   SELECT_COMMAND_REG;
   DISPLAY_ENABLE_HIGH;
   LCD_DATA_OUT &= ~DATA_LINE_BITMASK;	//clear only nibble which is connected to LCD, or both
   if (DATA_LINE_BITMASK == 0x0f)
      LCD_DATA_OUT |= ucData>>4; 		//remain high nibble untouched!
   else if (DATA_LINE_BITMASK == 0xf0)
      LCD_DATA_OUT |= ucData & 0xf0;	//remain low nibble untouched!
   else
      LCD_DATA_OUT |= ucData;			//use both nibbles
   vLCD_delay(1);
   DISPLAY_ENABLE_LOW;
   vLCD_delay(1);
}

//********************************************************************************
//Description  : initialize the LCD controller
//             : needs about 15ms to start
//Input        : lcd_columns	- count of columns
//Output       : none
//********************************************************************************
void vLCD_init(unsigned char ucLCDcolumns) {
//wait for more than 40ms after Vcc rises to 2.7V
//wait for more than 15ms after Vcc rises to 4.5V
	volatile unsigned int i;

	LCD_DATA_DIR |= DATA_LINE_BITMASK; 		//sets data lines as outputs
	LCD_COMM_DIR |= CONTROL_LINE_BITMASK;	//sets control lines as outputs
	SELECT_COMMAND_REG;
	DISPLAY_ENABLE_LOW;

	vLCD_delay(300);	//300x50us=15ms

	vSendByteToLCD_8BIT(0x30); //function set (Interface 8bit)
	vLCD_delay(82);	//82x50us=4.1ms
	vSendByteToLCD_8BIT(0x30); //function set (Interface 8bit)
	vLCD_delay(3);		//3x50us=150us
	vSendByteToLCD_8BIT(0x30); //function set (Interface 8bit)
	vLCD_delay(1);
	vSendByteToLCD_8BIT(0x20); //function set 4bit (Interface 8bit)
	vLCD_delay(1);
	vSendByteToLCD_4BIT(0x28, COMMAND); //function set + number of display lines + character font
	vLCD_delay(1);

	vLCD_clear(); //clear display
	vSendByteToLCD_4BIT(0x06, COMMAND); //entry mode set, Cursor moves, no Display shift
}

//********************************************************************************
// Description  : set the LCD display position x=1..16 y=1..2
//              :
// Input        : x - X-position
//              : y - Y-position
// Output       : none
//********************************************************************************
void vLCD_gotoxy(unsigned char ucX, unsigned char ucY) {
	vSendByteToLCD_4BIT(uiBase_y[ucY-1] + ucX-1, COMMAND);
}

//********************************************************************************
// Description  : clears the LCD
// Input        : none
// Output       : none
//********************************************************************************
void vLCD_clear(void) {
	volatile unsigned int i;

	vSendByteToLCD_4BIT(0x0c, COMMAND); //Display ON, Cursor and Blinking off
	vLCD_delay(1);
	vSendByteToLCD_4BIT(0x01, COMMAND); //function display clear
	vLCD_delay(40);			//40x50us=2ms, wait for clear operation (>=1.52ms)
	vLCD_gotoxy(1,1);

}

//********************************************************************************
// Description  : write a char to the LCD
// Input        : c - character
// Output       : none
//********************************************************************************
void vLCD_putc(unsigned char ucC) {
	vLCD_delay(1);
	vSendByteToLCD_4BIT(ucC, DATA);
}

//********************************************************************************
// Description  : write the string str to the LCD
// Input        : *str	pointer to string
// Output       : none
//********************************************************************************
void vLCD_puts(char *ucStr) {
	while (*ucStr != 0x0)
		vLCD_putc(*ucStr++);
}

// Hilfsfunktion: gibt eine Stelle auf dem LCD aus (fhrende Nullen werden ignoriert)
unsigned char put_uidigit(unsigned int *uiX, unsigned char stelle,
		unsigned char print) {
	unsigned int y, dezimal = 1;
	while (stelle) {
		dezimal = dezimal * 10;
		--stelle;
	}
	y = *uiX / dezimal;
	if (y || print) {
		vLCD_putc(y + 0x30);
		print = 1;
		*uiX -= y * dezimal;
	}
	return print;
}

//********************************************************************************
// Description  : write an unsigned integer to LCD
// Input        : unsigned integer uiX
// Output       : none
//********************************************************************************
// vorzeichenlose 16-Bit-Zahl als Dezimal-Wert auf dem Display ausgeben
void vLCD_putui(unsigned int uiX) {
	unsigned char stelle, print = 0;

	for (stelle = 4; stelle; --stelle) {
		print = put_uidigit(&uiX, stelle, print); // fhrende Nullen nicht ausgeben
	}
	vLCD_putc(uiX + 0x30); // die Einerstelle immer ausgeben
}


//********************************************************************************
// Description  : write a signed integer to LCD
// Input        : signed integer iX
// Output       : none
//********************************************************************************
void vLCD_puti(int iX) {
	if (iX < 0) {
		vLCD_putc('-');
		iX = ~iX + 1;
	}
	vLCD_putui((unsigned int) iX);
}

// zur Ausgabe eines long int auf dem Display
// *********************  Umwandlung von int bzw. long in ASCII-Zeichen  *********************
//
//  fhrende Nullen knnen unterdrckt werden: trim = 1, sonst trim = 0.
//  "Nachkommastellen" (hier als'.' gedruckt) knnen angegeben werden.
//  Es wird kein! String erzeugt. D.h. ein abschlieendes '\x00' wird nicht gesetzt.

// ***************  Prototypen; diese Funktionen werden nur intern verwendet  ***************
unsigned char l2chars(long value, char* asciiChars, unsigned char stellen,
		unsigned char komma, unsigned char trim);
char cldigit(long* uiX, unsigned char stelle, unsigned char* print);

// wandelt ein int in ASCII-Zeichen um.
//  der Pointer *asciiChars muss auf ein ausreichend groes Array (min. 7) zeigen!
//	Eine Null als Stringende wird nicht! gesetzt und ist dabei nicht mitgezhlt!
//  Komma gibt die Anzahl "Nachkommastellen" an, trim = 1: fhrende Nullen unterdrcken
//  Rckgabe: Anzahl gltiger Zeichen
unsigned char itoChars(int value, char* asciiChars, unsigned char komma,
		unsigned char trim) {
	return l2chars((long) value, asciiChars, 4, komma, trim);
}

// wandelt ein long int in ASCII-Zeichen um.
//  der Stringpointer *asciiChars muss auf ein ausreichend groes Array (min. 12) zeigen!
//	Eine Null als Stringende wird nicht! gesetzt und ist dabei nicht mitgezhlt!
//  Komma gibt die Anzahl "Nachkommastellen" an, trim = 1: fhrende Nullen unterdrcken
//  Rckgabe: Anzahl gltiger Zeichen
unsigned char ltoChars(long value, char* asciiChars, unsigned char komma,
		unsigned char trim) {
	return l2chars(value, asciiChars, 9, komma, trim);
}

//                         *** Hilfsfunktionen ***
//
char cldigit(long* uiX, unsigned char stelle, unsigned char* print) {
	unsigned long y, wert = 1;
	char data;
	while (stelle) {
		wert = wert * 10;
		--stelle;
	}
	y = *uiX / wert;
	if (y || *print) {
		data = y + 0x30;
		*uiX -= y * wert;
		*print = 1;
	} else
		data = ' ';
	return data;
}

unsigned char l2chars(long value, char* asciiChars, unsigned char nachkommastellen,
		unsigned char komma, unsigned char trim) {
	unsigned char print = 0;
	unsigned int index = 1;

	if (value < 0) { // Vorzeichen bearbeiten
		value = -value;
		asciiChars[0] = '-';
	} else {
		if (!trim) {
			asciiChars[0] = ' ';
		} else
			index = 0;
	}

	while (nachkommastellen) { // fr alle Stellen bis auf die Einerstelle
		asciiChars[index] = cldigit(&value, nachkommastellen, &print);
		if (komma && komma == nachkommastellen) {
			if (asciiChars[index] == ' ')
				asciiChars[index] = '0';
			asciiChars[++index] = '.';
			print = 1;
		}
		if (!print) {
			if (trim) {
				--index;
			} else {
				if (index > 0) {
					asciiChars[index] = asciiChars[index - 1]; // evtl. Vorzeichen mitziehen
					asciiChars[index - 1] = ' ';
				}
			}
		}
		++index;
		--nachkommastellen;

	}
	asciiChars[index] = value + 0x30; // Einerstelle immer schreiben
	return index + 1;
}

void vLCD_putli(long value, unsigned int nachkommestellen) {
	char cbuffer[15];

	cbuffer[ltoChars(value, cbuffer, nachkommestellen, 0)] = '\x00';
	vLCD_puts(cbuffer);
}



// Ende LCD.c
