/*
 * LSM9DS0.c
 *
 *  Created on: 25.08.2014
 *      Author: DL9MWE
 *
 * Details of the LSM9DS0: https://cdn.sparkfun.com/assets/f/6/1/f/0/LSM9DS0.pdf
 * and: http://www.st.com/st-web-ui/static/active/en/resource/technical/document/datasheet/DM00087365.pdf
 */

#include "main.h"

//t_lsm9ds0 lsm9ds0;

// I2C-Interface
unsigned char read9D_register_table15(unsigned char command) {
	return 0;
}

unsigned char read9D_register_table16(unsigned char command) {
	return 0;
}


int iLSM9DS0_init(void) {
	unsigned char who;
	unsigned int error_code;

	// device identification codes prfen
	who = ucI2C_read_8Bit_Register(gAddress, WHO_AM_I_G, &error_code);			// Device Identification, should read as 11010100
	if(who == 0xD4) {
		who = ucI2C_read_8Bit_Register(xmAddress, WHO_AM_I_G, &error_code);		// Device Identification, should read as 01001001
		if(who == 0x49) {					// alles ok
		// Gyro initialisieren
			/* CTRL_REG1_G sets output data rate, bandwidth, power-down and enables
			Bits[7:0]: DR1 DR0 BW1 BW0 PD Zen Xen Yen
			DR[1:0] - Output data rate selection
				00=95Hz, 01=190Hz, 10=380Hz, 11=760Hz
			BW[1:0] - Bandwidth selection (sets cutoff frequency)
				 Value depends on ODR. See datasheet table 21.
			PD - Power down enable (0=power down mode, 1=normal or sleep mode)
			Zen, Xen, Yen - Axis enable (o=disabled, 1=enabled)	*/
			error_code = iI2C_write_register_value(gAddress, CTRL_REG1_G, 0x0F); // Normal mode, enable all axes

			/* CTRL_REG2_G sets up the HPF
			Bits[7:0]: 0 0 HPM1 HPM0 HPCF3 HPCF2 HPCF1 HPCF0
			HPM[1:0] - High pass filter mode selection
				00=normal (reset reading HP_RESET_FILTER, 01=ref signal for filtering,
				10=normal, 11=autoreset on interrupt
			HPCF[3:0] - High pass filter cutoff frequency
				Value depends on data rate. See datasheet table 26.
			*/
			error_code = iI2C_write_register_value(gAddress, CTRL_REG2_G, 0x00); // Normal mode, high cutoff frequency

			/* CTRL_REG3_G sets up interrupt and DRDY_G pins
			Bits[7:0]: I1_IINT1 I1_BOOT H_LACTIVE PP_OD I2_DRDY I2_WTM I2_ORUN I2_EMPTY
			I1_INT1 - Interrupt enable on INT_G pin (0=disable, 1=enable)
			I1_BOOT - Boot status available on INT_G (0=disable, 1=enable)
			H_LACTIVE - Interrupt active configuration on INT_G (0:high, 1:low)
			PP_OD - Push-pull/open-drain (0=push-pull, 1=open-drain)
			I2_DRDY - Data ready on DRDY_G (0=disable, 1=enable)
			I2_WTM - FIFO watermark interrupt on DRDY_G (0=disable 1=enable)
			I2_ORUN - FIFO overrun interrupt on DRDY_G (0=disable 1=enable)
			I2_EMPTY - FIFO empty interrupt on DRDY_G (0=disable 1=enable) */
			// Int1 enabled (pp, active low), data read on DRDY_G:
			error_code = iI2C_write_register_value(gAddress, CTRL_REG3_G, 0x88);

			/* CTRL_REG4_G sets the scale, update mode
			Bits[7:0] - BDU BLE FS1 FS0 - ST1 ST0 SIM
			BDU - Block data update (0=continuous, 1=output not updated until read
			BLE - Big/little endian (0=data LSB @ lower address, 1=LSB @ higher add)
			FS[1:0] - Full-scale selection
				00=245dps, 01=500dps, 10=2000dps, 11=2000dps
			ST[1:0] - Self-test enable
				00=disabled, 01=st 0 (x+, y-, z-), 10=undefined, 11=st 1 (x-, y+, z+)
			SIM - SPI serial interface mode select
				0=4 wire, 1=3 wire */
			error_code = iI2C_write_register_value(gAddress, CTRL_REG4_G, 0x00); // Set scale to 245 dps

			/* CTRL_REG5_G sets up the FIFO, HPF, and INT1
			Bits[7:0] - BOOT FIFO_EN - HPen INT1_Sel1 INT1_Sel0 Out_Sel1 Out_Sel0
			BOOT - Reboot memory content (0=normal, 1=reboot)
			FIFO_EN - FIFO enable (0=disable, 1=enable)
			HPen - HPF enable (0=disable, 1=enable)
			INT1_Sel[1:0] - Int 1 selection configuration
			Out_Sel[1:0] - Out selection configuration */
			error_code = iI2C_write_register_value(gAddress, CTRL_REG5_G, 0x00);


		// Beschleunigungs-Sensor und Magnetfeld-Sensor initialisieren
			/* CTRL_REG0_XM (0x1F) (Default value: 0x00)
			Bits (7-0): BOOT FIFO_EN WTM_EN 0 0 HP_CLICK HPIS1 HPIS2
			BOOT - Reboot memory content (0: normal, 1: reboot)
			FIFO_EN - Fifo enable (0: disable, 1: enable)
			WTM_EN - FIFO watermark enable (0: disable, 1: enable)
			HP_CLICK - HPF enabled for click (0: filter bypassed, 1: enabled)
			HPIS1 - HPF enabled for interrupt generator 1 (0: bypassed, 1: enabled)
			HPIS2 - HPF enabled for interrupt generator 2 (0: bypassed, 1 enabled)   */
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG0_XM, 0x00);

			/* CTRL_REG1_XM (0x20) (Default value: 0x07)
			Bits (7-0): AODR3 AODR2 AODR1 AODR0 BDU AZEN AYEN AXEN
			AODR[3:0] - select the acceleration data rate:
				0000=power down, 0001=3.125Hz, 0010=6.25Hz, 0011=12.5Hz,
				0100=25Hz, 0101=50Hz, 0110=100Hz, 0111=200Hz, 1000=400Hz,
				1001=800Hz, 1010=1600Hz, (remaining combinations undefined).
			BDU - block data update for accel AND mag
				0: Continuous update
				1: Output registers aren't updated until MSB and LSB have been read.
			AZEN, AYEN, and AXEN - Acceleration x/y/z-axis enabled.
				0: Axis disabled, 1: Axis enabled									 */
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG1_XM, 0x57); // 100Hz data rate, x/y/z all enabled

			/* CTRL_REG2_XM (0x21) (Default value: 0x00)
			Bits (7-0): ABW1 ABW0 AFS2 AFS1 AFS0 AST1 AST0 SIM
			ABW[1:0] - Accelerometer anti-alias filter bandwidth
				00=773Hz, 01=194Hz, 10=362Hz, 11=50Hz
			AFS[2:0] - Accel full-scale selection
				000=+/-2g, 001=+/-4g, 010=+/-6g, 011=+/-8g, 100=+/-16g
			AST[1:0] - Accel self-test enable
				00=normal (no self-test), 01=positive st, 10=negative st, 11=not allowed
			SIM - SPI mode selection
				0=4-wire, 1=3-wire													 */
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG2_XM, 0x08); // Set scale to 4g

			/* CTRL_REG3_XM is used to set interrupt generators on INT1_XM
			Bits (7-0): P1_BOOT P1_TAP P1_INT1 P1_INT2 P1_INTM P1_DRDYA P1_DRDYM P1_EMPTY
			*/
			// Accelerometer data ready on INT1_XM (0x04)
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG3_XM, 0x04);

		// init Mag
			/* CTRL_REG5_XM enables temp sensor, sets mag resolution and data rate
			Bits (7-0): TEMP_EN M_RES1 M_RES0 M_ODR2 M_ODR1 M_ODR0 LIR2 LIR1
			TEMP_EN - Enable temperature sensor (0=disabled, 1=enabled)
			M_RES[1:0] - Magnetometer resolution select (0=low, 3=high)
			M_ODR[2:0] - Magnetometer data rate select
				000=3.125Hz, 001=6.25Hz, 010=12.5Hz, 011=25Hz, 100=50Hz, 101=100Hz
			LIR2 - Latch interrupt request on INT2_SRC (cleared by reading INT2_SRC)
				0=interrupt request not latched, 1=interrupt request latched
			LIR1 - Latch interrupt request on INT1_SRC (cleared by readging INT1_SRC)
				0=irq not latched, 1=irq latched 									 */
//			vI2C_write_register_value(xmAddress, CTRL_REG5_XM, 0x14, &error_code); // Mag data rate - 100 Hz
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG5_XM, 0x94); // Temp enabled, Mag data rate - 100 Hz

			/* CTRL_REG6_XM sets the magnetometer full-scale
			Bits (7-0): 0 MFS1 MFS0 0 0 0 0 0
			MFS[1:0] - Magnetic full-scale selection
			00:+/-2Gauss, 01:+/-4Gs, 10:+/-8Gs, 11:+/-12Gs							 */
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG6_XM, 0x00); // Mag scale to +/- 2GS

			/* CTRL_REG7_XM sets magnetic sensor mode, low power mode, and filters
			AHPM1 AHPM0 AFDS 0 0 MLP MD1 MD0
			AHPM[1:0] - HPF mode selection
				00=normal (resets reference registers), 01=reference signal for filtering,
				10=normal, 11=autoreset on interrupt event
			AFDS - Filtered acceleration data selection
				0=internal filter bypassed, 1=data from internal filter sent to FIFO
			MLP - Magnetic data low-power mode
				0=data rate is set by M_ODR bits in CTRL_REG5
				1=data rate is set to 3.125Hz
			MD[1:0] - Magnetic sensor mode selection (default 10)
				00=continuous-conversion, 01=single-conversion, 10 and 11=power-down */
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG7_XM, 0x00); // Continuous conversion mode

			/* CTRL_REG4_XM is used to set interrupt generators on INT2_XM
			Bits (7-0): P2_TAP P2_INT1 P2_INT2 P2_INTM P2_DRDYA P2_DRDYM P2_Overrun P2_WTM
			*/
			error_code = iI2C_write_register_value(xmAddress, CTRL_REG4_XM, 0x04); // Magnetometer data ready on INT2_XM (0x08)

			/* INT_CTRL_REG_M to set push-pull/open drain, and active-low/high
			Bits[7:0] - XMIEN YMIEN ZMIEN PP_OD IEA IEL 4D MIEN
			XMIEN, YMIEN, ZMIEN - Enable interrupt recognition on axis for mag data
			PP_OD - Push-pull/open-drain interrupt configuration (0=push-pull, 1=od)
			IEA - Interrupt polarity for accel and magneto
				0=active-low, 1=active-high
			IEL - Latch interrupt request for accel and magneto
				0=irq not latched, 1=irq latched
			4D - 4D enable. 4D detection is enabled when 6D bit in INT_GEN1_REG is set
			MIEN - Enable interrupt generation for magnetic data
				0=disable, 1=enable) */
			error_code = iI2C_write_register_value(xmAddress, INT_CTRL_REG_M, 0x09); // Enable interrupts for mag, active-low, push-pull

			return 0;	// Ok
		}
	}
	return 1;
}

// Durchschnittswert aus 8 Messungen bilden
int average_8(int data[8]) {
	long summe=0;
	unsigned int index = 8-1;
	do {
		summe += data[index];	// Summe der 8 Werte ermitteln
	} while(index--);
	return (int)(summe >> 3);	// Summe durch 8 =^ Durchschnitt
}

// X, Y und Z-Werte ber 8 Messungen mitteln
int iread_3axis(unsigned char I2C_DeviceAddress, unsigned int sub_address, int *x, int* y, int *z) {
	unsigned char status_reg;
	if(I2C_DeviceAddress == xmAddress) {
		if(sub_address == OUT_X_L_A)	status_reg = STATUS_REG_A;
		else							status_reg = STATUS_REG_M;
	}	else							status_reg = STATUS_REG_G;

	unsigned int error_code;
	unsigned int index;
	unsigned char rxData[6];
	int dataX[8], dataY[8], dataZ[8];
	for(index = 0; index < 8; ++index) {
		while(!(ucI2C_read_8Bit_Register(I2C_DeviceAddress, status_reg, &error_code) & BIT4));	 // wait for data ready
		error_code = iI2C_read_multiple_chars(I2C_DeviceAddress, sub_address, 6, rxData);
		dataX[index] = (int)rxData[0] | ((int)rxData[1] << 8);	// 16-Bit Werte
		dataY[index] = (int)rxData[2] | ((int)rxData[3] << 8);	 // aus je zwei Bytes
		dataZ[index] = (int)rxData[4] | ((int)rxData[5] << 8);	  // zusammen setzen
	}
	*x = average_8(dataX);
	*y = average_8(dataY);
	*z = average_8(dataZ);
	return error_code;
}

// Temperaturwert auslesen
int iread_temp(int *temp) {
	unsigned char rxData[2];
	int error_code;
	error_code = iI2C_read_multiple_chars(xmAddress, OUT_TEMP_L_XM, 2, rxData);
	*temp = (int)rxData[0] | ((int)rxData[1] << 8);
	return error_code;
}

// Ende LSM9DS0.c
