/*
 * main.c
 * LSM9DS0 (Sensor fr lineare Beschleunigung, Drehrate und Magnetfeld sowie Temperatur)und
 * BMP180  (Sensor fr Luftdruck und Temperatur)
 *  via I2C am MSP430G2553
 *  02.09.2014 DL9MWE   pse qsl
 */
#include "main.h"

// interne Prototypes
void vdisplay_value_(int value, int x, int y, char disp);
void vdisplay_xyz(int x, int y, int z, char *disp);
void vdisplay_temp(void);
void vSensor_error(char *disp);
int taster(void);


int main(void) {
	vCPU_init();
	vI2C_init();
	vLCD_init(16);
	while(iLSM9DS0_init())	{	// Gyro: +-245dps, Beschleunigung: +-4g, Magnetfeld: +-2 Gauss
		vSensor_error("LSM9DS0");		// Sensor nicht erkannt
	}
	while(iBMP180_init()) {
		vSensor_error("BMP180");		// Sensor nicht erkannt
	}

	const messung_t messungen[3] = {
			{GYRO, "gyr"},				// Drehbewegung in x, y, z-Richtung
			{MAGNETFELD, "mag"},		// Magnetfeld-Komponenten
			{BESCHLEUNIGUNG, "acc"}		// lineare Beschleunigungen
	};
	unsigned int messung_index=0;
	int temp;
	long int luftdruck;
	unsigned int error_code=0;

	while(42) {
		switch(messung_index) {
		case 0:
		case 1:
		case 2:
			while(1) {		// 3D-Werte aus dem LSM9DS0 lesen und anzeigen
				int x, y, z;
				if(iread_3axis(messungen[messung_index].I2C_device_address, messungen[messung_index].I2C_register, &x, &y, &z)) {
					vSensor_error("LSM9DS0");	// dumm gelaufen; der Sensor ist per I2C nicht erreichbar
				} else {
					vdisplay_xyz(x, y, z, messungen[messung_index].disp);
					vdisplay_temp();
				}
				if(taster())	break;
			}
			++messung_index;
			break;
		case 3:				// Luftdruck-Sensor BMP180
			while(1) {
				temp = iBMP180_get_temperature(&error_code);	// Temperaturwert ermitteln
				luftdruck = liBMP180_get_pressure(&error_code);
				vLCD_clear();
				vLCD_puts("BMP180");
				vLCD_gotoxy(1,2);
				if(error_code) {
					vSensor_error("BMP180");	// dumm gelaufen; der Sensor ist per I2C nicht erreichbar
				} else {
					vLCD_puts("QFE ");
					vLCD_putli(luftdruck, 2);
					vLCD_gotoxy(9,1);
					vLCD_puts("T=");
					vLCD_puti(temp);
					_delay_ms(200);
				}
				if(taster())	break;
			}
			messung_index = 0;	// weiter mit LSM9DS0
			break;
		}
	}
}


// interne Funktionen
void vCPU_init(void) {
	WDTCTL = WDTPW + WDTHOLD; //stoppe Watchdog Timer
	//initialize Basic Clock Module
	// fr MCLK wird der interne Oszillator wird verwendet	=> Set DCO to 1 MHz
	DCOCTL = 0;								// Select lowest DCOx and MODx settings
	BCSCTL1 = CALBC1_1MHZ;					// Set range
	DCOCTL= CALDCO_1MHZ;					// Set DCO step + modulation
	BCSCTL2 = SELM_0 | DIVM_0 | DIVS_0;
}

// msec Millisekunden warten
void _delay_ms(unsigned int msec) {
  volatile unsigned int count;

	while(--msec) {
		count = 100;	// passt bei 1MHz aus dem internen Oszillator DCO
		while(count) {
			--count;
		}
	}
}

//	Werte auf dem Display darstellen
void vdisplay_value_(int value, int x, int y, char disp) {
	vLCD_gotoxy(x,y);
	vLCD_putc(disp);
	vLCD_putc('=');
	vLCD_puti(value);
	vLCD_puts("     ");
}
void vdisplay_xyz(int x, int y, int z, char *disp) {
	vdisplay_value_(x, 1, 1, 'X');
	vdisplay_value_(y, 9, 1, 'Y');
	vdisplay_value_(z, 1, 2, 'Z');
	vLCD_gotoxy(9,2);
	vLCD_puts(disp);
}
void vdisplay_temp(void) {
	int temp;
	vLCD_gotoxy(12,2);
	if(iread_temp(&temp)) {
		vLCD_puts("T=?");
	} else {
		vLCD_puts("T=");
		vLCD_puti(210+(temp));
		vLCD_putc(' ');
	}
}

void vSensor_error(char *sensor) {
	vLCD_clear();
	vLCD_puts(sensor);
	vLCD_gotoxy(1,2);
	vLCD_puts("Sensor I2C-Error");		//  9-Axis-Sensor LSM9DS0 nicht erkannt
	_delay_ms(500);
}

int taster(void) {
	if(!(P1IN & BIT3)) {	// LaunchPad-Taster an P1.3 schaltet weiter zum nchsten Messwert
		vLCD_gotoxy(9,2);
		vLCD_puts("???");
		while(!(P1IN & BIT3))
			_delay_ms(10);	// bitte Taster wieder loslassen
		return 1;
	}
	return 0;
}


// Ende main.c
